import torch
from transformers import LlamaTokenizer, LlamaForCausalLM
from translang import TranslationService


class LLMtranslator(TranslationService):
    """
    LLMtranslator is a translation service based on the Llama language model.
    It extends the TranslationService class and provides translation functionality using Llama.

    Args:
        model_path (str): The path to the Llama model.
        target_lang (str, optional): The target language for translation (default: "en").
        translator (str, optional): The translation service to use (default: "google").
        torch_dtype (torch.dtype, optional): The data type for torch (default: torch.float16).
        device_map (str, optional): The device map for torch (default: "auto").
        deepl_api_key (str, optional): DeepL API key (if required).
        bard_api_key (str, optional): Bard API key (if required).
        openai_model (str, optional): OpenAI model name (default: "gpt-3.5-turbo").
        openai_api_key (str, optional): OpenAI API key (if required).
    """

    def __init__(
        self,
        model_path,
        target_lang="en",
        translator="google",
        torch_dtype=torch.float16,
        offload_folder=None,
        device_map="auto",
        google_api_key=None,
        deepl_api_key=None,
        bard_api_key=None,
        openai_api_key=None,
        openai_model="gpt-3.5-turbo",
    ):
        super().__init__(
            translator=translator,
            google_api_key=google_api_key,
            deepl_api_key=deepl_api_key,
            bard_api_key=bard_api_key,
            openai_api_key=openai_api_key,
            openai_model=openai_model,
        )
        self.model_path = model_path
        self.target_lang = target_lang
        self.tokenizer = LlamaTokenizer.from_pretrained(model_path)
        self.model = LlamaForCausalLM.from_pretrained(
            model_path, torch_dtype=torch_dtype, device_map=device_map, offload_folder=offload_folder
        )
        self.model.tie_weights()

    def process_prompt(self, prompt: str) -> str:
        """
        Preprocesses the prompt text for translation.

        Args:
            prompt (str): The prompt text.

        Returns:
            str: The preprocessed prompt text.
        """
        return self.translate(prompt, "en")

    def process_answer(self, answer: str) -> str:
        """
        Postprocesses the translated answer.

        Args:
            answer (str): The translated answer.

        Returns:
            str: The postprocessed answer.
        """
        return self.translate(answer, self.target_lang)

    def inference(self, prompt: str) -> str:
        """
        Generates the translated answer based on the given prompt.

        Args:
            prompt (str): The prompt text.

        Returns:
            str: The translated answer.
        """
        input_ids = self.tokenizer(prompt, return_tensors="pt").input_ids
        
        # Different from this class's generate method is the generate method of the Hugging Face model.
        with torch.no_grad():
            generation_output = self.model.generate(input_ids=input_ids, max_length=32) 
        answer = self.tokenizer.decode(generation_output[0])
        return answer

    def generate(self, prompt: str) -> str:
        """
        Generates the translated answer for the given prompt.

        Args:
            prompt (str): The prompt text.

        Returns:
            str: The translated answer.
        """
        if self.target_lang =='en':
            return self.inference(prompt)
        else:
            translated_prompt = self.process_prompt(prompt)
            answer = self.inference(translated_prompt)
            translated_answer = self.process_answer(answer)
        return translated_answer



import re

from setuptools import find_packages
from setuptools import setup


def get_version():
    filename = "transllm/__init__.py"
    with open(filename) as f:
        match = re.search(r"""^__version__ = ['"]([^'"]*)['"]""", f.read(), re.M)
    if not match:
        raise RuntimeError("{} doesn't contain __version__".format(filename))
    version = match.groups()[0]
    return version


def get_long_description():
    with open("README.md", encoding="UTF-8") as f:
        long_description = f.read()
        return long_description


version = get_version()


setup(
    name="transllm",
    version="0.1.8",
    author="daniel park",
    author_email="parkminwoo1991@gmail.com",
    description="LLMtranslator translates and generates text in multiple languages.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/dsdanielpark/hf-transllm",
    packages=find_packages(exclude=[]),
    python_requires=">=3.6",
    install_requires=[
        "requests",
        "deep_translator",
        "colorama",
        "httpx[http2]",
        "torch",
        "deepl",
        "transformers",
        "bardapi",
        "openai",
        "sentencepiece",
        "accelerate",
        "translang"
    ],
    keywords="Python, API, Bard, Google Bard, Large Language Model, Chatbot API, Google API, Chatbot",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    entry_points={"console_scripts": ["transllm=transllm.cli:main"]},
)
